@@ -1,9 +1,9 @@
 package gnu.crypto.util;
 
 // ----------------------------------------------------------------------------
-// $Id: Util.java,v 1.7 2002/12/30 05:36:49 rsdio Exp $
+// $Id: Util.java,v 1.10 2003/09/27 00:03:01 raif Exp $
 //
-// Copyright (C) 2001, 2002, Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
 //
 // This file is part of GNU Crypto.
 //
@@ -43,19 +43,25 @@ package gnu.crypto.util;
 // do so, delete this exception statement from your version.
 // ----------------------------------------------------------------------------
 
+import java.math.BigInteger;
+
 /**
  * <p>A collection of utility methods used throughout this project.</p>
  *
- * @version $Revision: 1.7 $
+ * @version $Revision: 1.10 $
  */
 public class Util {
 
    // Constants and variables
    // -------------------------------------------------------------------------
 
-   private static final char[] HEX_DIGITS = {
-      '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
-   };
+   // Hex charset
+   private static final char[] HEX_DIGITS = "0123456789ABCDEF".toCharArray();
+
+   // Base-64 charset
+   private static final String BASE64_CHARS =
+      "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz./";
+   private static final char[] BASE64_CHARSET = BASE64_CHARS.toCharArray();
 
    // Constructor(s)
    // -------------------------------------------------------------------------
@@ -68,31 +74,6 @@ public class Util {
    // Class methods
    // -------------------------------------------------------------------------
 
-   /**
-    * <p>Returns <code>true</code> if the two designated byte arrays are
-    * (a) non-null, (b) of the same length, and (c) contain the same values.</p>
-    *
-    * @param a the first byte array.
-    * @param b the second byte array.
-    * @return <code>true</code> if the two designated arrays contain the same
-    * values. Returns <code>false</code> otherwise.
-    */
-   public static boolean areEqual(byte[] a, byte[] b) {
-      if (a == null || b == null) {
-         return false;
-      }
-      int aLength = a.length;
-      if (aLength != b.length) {
-         return false;
-      }
-      for (int i = 0; i < aLength; i++) {
-         if (a[i] != b[i]) {
-            return false;
-         }
-      }
-      return true;
-   }
-
    /**
     * <p>Returns a string of hexadecimal digits from a byte array. Each byte is
     * converted to 2 hex symbols; zero(es) included.</p>
@@ -114,7 +95,7 @@ public class Util {
    /**
     * <p>Returns a string of hexadecimal digits from a byte array, starting at
     * <code>offset</code> and consisting of <code>length</code> bytes. Each byte
-    * is converted to 2 hex symbols; zero(es) included.
+    * is converted to 2 hex symbols; zero(es) included.</p>
     *
     * @param ba the byte array to convert.
     * @param offset the index from which to start considering the bytes to
@@ -200,11 +181,11 @@ public class Util {
       }
       return result;
    }
-   
+
    /**
-    * <p>Returns a byte array from a string of hexadecimal digits,
-    * interpreting them as a large big-endian integer and returning it
-    * as a large little-endian integer.</p>
+    * <p>Returns a byte array from a string of hexadecimal digits, interpreting
+    * them as a large big-endian integer and returning it as a large
+    * little-endian integer.</p>
     *
     * @param s a string of hexadecimal ASCII characters
     * @return the decoded byte array from the input hexadecimal string.
@@ -278,8 +259,8 @@ public class Util {
    }
 
    /**
-    * <p>Returns a string of 16 hexadecimal digits (most significant digit
-    * first) corresponding to the unsigned long <code>n</code>.</p>
+    * <p>Returns a string of 16 hexadecimal digits (most significant digit first)
+    * corresponding to the unsigned long <code>n</code>.</p>
     *
     * @param n the unsigned long to convert.
     * @return a hexadecimal string 16-character long.
@@ -299,7 +280,7 @@ public class Util {
     * externalise byte arrays that will be constructed later from such strings;
     * eg. s-box values.</p>
     *
-    * @exception ArrayIndexOutOfBoundsException if the length is odd.
+    * @throws ArrayIndexOutOfBoundsException if the length is odd.
     */
    public static String toUnicodeString(byte[] ba) {
       return toUnicodeString(ba, 0, ba.length);
@@ -311,7 +292,7 @@ public class Util {
     * externalise byte arrays that will be constructed later from such strings;
     * eg. s-box values.</p>
     *
-    * @exception ArrayIndexOutOfBoundsException if the length is odd.
+    * @throws ArrayIndexOutOfBoundsException if the length is odd.
     */
    public static final String
    toUnicodeString(byte[] ba, int offset, int length) {
@@ -345,8 +326,7 @@ public class Util {
     * externalise integer arrays that will be constructed later from such
     * strings; eg. s-box values.</p>
     *
-    * @exception ArrayIndexOutOfBoundsException if the length is not a multiple
-    * of 4.
+    * @throws ArrayIndexOutOfBoundsException if the length is not a multiple of 4.
     */
    public static String toUnicodeString(int[] ia) {
       StringBuffer sb = new StringBuffer();
@@ -383,10 +363,250 @@ public class Util {
          c = s.charAt(i >>> 1);
          result[i] = (byte)(((i & 1) == 0) ? c >>> 8 : c);
       }
-
       return result;
    }
 
-   // Instance methods
-   // -------------------------------------------------------------------------
+   /**
+    * <p>Dumps a byte array as a string, in a format that is easy to read for
+    * debugging. The string <code>m</code> is prepended to the start of each
+    * line.</p>
+    *
+    * <p>If <code>offset</code> and <code>length</code> are omitted, the whole
+    * array is used. If <code>m</code> is omitted, nothing is prepended to each
+    * line.</p>
+    *
+    * @param data the byte array to be dumped.
+    * @param offset the offset within <i>data</i> to start from.
+    * @param length the number of bytes to dump.
+    * @param m a string to be prepended to each line.
+    * @return a string containing the result.
+    */
+   public static String dumpString(byte[] data, int offset, int length, String m) {
+      if (data == null) {
+         return m + "null\n";
+      }
+      StringBuffer sb = new StringBuffer(length * 3);
+      if (length > 32) {
+         sb.append(m).append("Hexadecimal dump of ").append(length).append(" bytes...\n");
+      }
+      // each line will list 32 bytes in 4 groups of 8 each
+      int end = offset + length;
+      String s;
+      int l = Integer.toString(length).length();
+      if (l < 4) {
+         l = 4;
+      }
+      for ( ; offset < end; offset += 32) {
+         if (length > 32) {
+            s = "         " + offset;
+            sb.append(m).append(s.substring(s.length()-l)).append(": ");
+         }
+         int i = 0;
+         for ( ; i < 32 && offset + i + 7 < end; i += 8) {
+            sb.append(toString(data, offset + i, 8)).append(' ');
+         }
+         if (i < 32) {
+            for ( ; i < 32 && offset + i < end; i++) {
+               sb.append(byteToString(data[offset + i]));
+            }
+         }
+         sb.append('\n');
+      }
+      return sb.toString();
+   }
+
+   public static String dumpString(byte[] data) {
+      return (data == null) ? "null\n" : dumpString(data, 0, data.length, "");
+   }
+
+   public static String dumpString(byte[] data, String m) {
+      return (data == null) ? "null\n" : dumpString(data, 0, data.length, m);
+   }
+
+   public static String dumpString(byte[] data, int offset, int length) {
+      return dumpString(data, offset, length, "");
+   }
+
+   /**
+    * <p>Returns a string of 2 hexadecimal digits (most significant digit first)
+    * corresponding to the lowest 8 bits of <code>n</code>.</p>
+    *
+    * @param n the byte value to convert.
+    * @return a string of 2 hex characters representing the input.
+    */
+   public static String byteToString(int n) {
+      char[] buf = { HEX_DIGITS[(n >>> 4) & 0x0F], HEX_DIGITS[n & 0x0F] };
+      return new String(buf);
+   }
+
+   /**
+    * <p>Converts a designated byte array to a Base-64 representation, with the
+    * exceptions that (a) leading 0-byte(s) are ignored, and (b) the character
+    * '.' (dot) shall be used instead of "+' (plus).</p>
+    *
+    * <p>Used by SASL password file manipulation primitives.</p>
+    *
+    * @param buffer an arbitrary sequence of bytes to represent in Base-64.
+    * @return unpadded (without the '=' character(s)) Base-64 representation of
+    * the input.
+    */
+   public static final String toBase64(byte[] buffer) {
+      int len = buffer.length, pos = len % 3;
+      byte b0 = 0, b1 = 0, b2 = 0;
+      switch (pos) {
+      case 1:
+         b2 = buffer[0];
+         break;
+      case 2:
+         b1 = buffer[0];
+         b2 = buffer[1];
+         break;
+      }
+      StringBuffer sb = new StringBuffer();
+      int c;
+      boolean notleading = false;
+      do {
+         c = (b0 & 0xFC) >>> 2;
+         if (notleading || c != 0) {
+           sb.append(BASE64_CHARSET[c]);
+           notleading = true;
+         }
+         c = ((b0 & 0x03) << 4) | ((b1 & 0xF0) >>> 4);
+         if (notleading || c != 0) {
+           sb.append(BASE64_CHARSET[c]);
+           notleading = true;
+         }
+         c = ((b1 & 0x0F) << 2) | ((b2 & 0xC0) >>> 6);
+         if (notleading || c != 0) {
+           sb.append(BASE64_CHARSET[c]);
+           notleading = true;
+         }
+         c = b2 & 0x3F;
+         if (notleading || c != 0) {
+           sb.append(BASE64_CHARSET[c]);
+           notleading = true;
+         }
+         if (pos >= len) {
+           break;
+         } else {
+           try {
+             b0 = buffer[pos++];
+             b1 = buffer[pos++];
+             b2 = buffer[pos++];
+           } catch (ArrayIndexOutOfBoundsException x) {
+             break;
+           }
+         }
+      } while (true);
+
+      if (notleading) {
+        return sb.toString();
+      }
+      return "0";
+   }
+
+   /**
+    * <p>The inverse function of the above.</p>
+    *
+    * <p>Converts a string representing the encoding of some bytes in Base-64
+    * to their original form.</p>
+    *
+    * @param str the Base-64 encoded representation of some byte(s).
+    * @return the bytes represented by the <code>str</code>.
+    * @throws NumberFormatException if <code>str</code> is <code>null</code>, or
+    * <code>str</code> contains an illegal Base-64 character.
+    * @see #toBase64(byte[])
+    */
+   public static final byte[] fromBase64(String str) {
+      int len = str.length();
+      if (len == 0) {
+         throw new NumberFormatException("Empty string");
+      }
+      byte[] a = new byte[len + 1];
+      int i, j;
+      for (i = 0; i < len; i++) {
+         try {
+            a[i] = (byte) BASE64_CHARS.indexOf(str.charAt(i));
+         } catch (ArrayIndexOutOfBoundsException x) {
+            throw new NumberFormatException("Illegal character at #"+i);
+         }
+      }
+      i = len - 1;
+      j = len;
+      try {
+         while (true) {
+            a[j] = a[i];
+            if (--i < 0) {
+               break;
+            }
+            a[j] |= (a[i] & 0x03) << 6;
+            j--;
+            a[j] = (byte)((a[i] & 0x3C) >>> 2);
+            if (--i < 0) {
+               break;
+            }
+            a[j] |= (a[i] & 0x0F) << 4;
+            j--;
+            a[j] = (byte)((a[i] & 0x30) >>> 4);
+            if (--i < 0) {
+               break;
+            }
+            a[j] |= (a[i] << 2);
+            j--;
+            a[j] = 0;
+            if (--i < 0) {
+               break;
+            }
+         }
+      } catch (Exception ignored) {
+      }
+
+      try { // ignore leading 0-bytes
+         while(a[j] == 0) {
+            j++;
+         }
+      } catch (Exception x) {
+         return new byte[1]; // one 0-byte
+      }
+      byte[] result = new byte[len - j + 1];
+      System.arraycopy(a, j, result, 0, len - j + 1);
+      return result;
+   }
+
+   // BigInteger utilities ----------------------------------------------------
+
+   /**
+    * <p>Treats the input as the MSB representation of a number, and discards
+    * leading zero elements. For efficiency, the input is simply returned if no
+    * leading zeroes are found.</p>
+    *
+    * @param n the {@link BigInteger} to trim.
+    * @return the byte array representation of the designated {@link BigInteger}
+    * with no leading 0-bytes.
+    */
+   public static final byte[] trim(BigInteger n) {
+      byte[] in = n.toByteArray();
+      if (in.length == 0 || in[0] != 0) {
+         return in;
+      }
+      int len = in.length;
+      int i = 1;
+      while (in[i] == 0 && i < len) {
+         ++i;
+      }
+      byte[] result = new byte[len - i];
+      System.arraycopy(in, i, result, 0, len - i);
+      return result;
+   }
+
+   /**
+    * <p>Returns a hexadecimal dump of the trimmed bytes of a {@link BigInteger}.
+    * </p>
+    *
+    * @param x the {@link BigInteger} to display.
+    * @return the string representation of the designated {@link BigInteger}.
+    */
+   public static final String dump(BigInteger x) {
+      return dumpString(trim(x));
+   }
 }
